module Autoaudio
  class Download < Thor
    include Autoaudio::Utils

    desc 'download_releases LOGGER, DOWNLOAD ATTEMPTS', 'download nfo and nzb files LOGGER, DOWNLOAD ATTEMPTS'
    def download_releases(logger="nolog", release_nzb_download_attempts=1)
      if logger == "log"
        if yes?("Do you want to wipe the old .log file? [yN]")
          File.truncate('./log/agent.log', 0)
        end
      end
      Make.new.releases
      Release.where(release_nzb_downloaded: false, :release_nzb_download_attempts.lte => release_nzb_download_attempts).each do |release|
        with_retries(:max_tries => 20, :base_sleep_seconds => 0.1, :max_sleep_seconds => 6.0) do
          make_release(logger, release)
        end
      end
      gist_tweet_releases if Settings.share_files
      say ""
      say "Thank you. Come again.", :white
    end

    no_commands do
      def make_release(logger, release)
        hash_regex          = Regexp.new('\b[A-Za-z0-9]{32}\b')
        psydealer_regex     = Regexp.new('PsyDealer')
        town_regex          = Regexp.new('town\.ag')
        flac_regex          = Regexp.new('a\.b\.flac|a\.b\.sounds\.flac|a\.b\.flacEFNet')

        download_location   = "#{Settings.download_location}/#{release.release_genre_formatted}/#{sanitize_filename(release.release_name)}"
        nzb                 = "#{download_location}/#{sanitize_filename(release.release_name)}.nzb"
        nfo                 = "#{download_location}/#{sanitize_filename(release.release_name)}.nfo"
        zip                 = "#{download_location}/#{sanitize_filename(release.release_name)}.zip"

        agent = Mechanize.new
        agent.user_agent_alias = 'Mac Safari'
        agent.log = Logger.new('./log/agent.log') if logger == "log"
        cookie = HTTP::Cookie.new(
          :domain  => 'nzbindex.nl',
          :name    => "agreed",
          :value   => "true",
          :path    => '/',
          :expires => (Date.today + 1).to_s)
        agent.cookie_jar.add(cookie)

        page          = agent.get('http://nzbindex.nl/')
        search_form   = page.form
        search_form.q = release.release_name
        results_page  = agent.submit(search_form, search_form.buttons.first)

        if !release.release_nzb_downloaded && results_page.links_with(:text => 'Download', :href => hash_regex).any? do |link|
          search_term = hash_regex.match(link.href)
          page        = agent.get("http://nzbindex.nl/groups/alt.binaries.inner-sanctum/?q=#{search_term}")
          form        = page.form_with(:action => 'http://nzbindex.nl/download/')
          form.checkboxes.each { |checkbox| checkbox.check if hash_regex.match(checkbox.text) }
          form.submit.save! nzb
          say "inner-sanctum => #{hash_regex.match(link.href)} = #{release.release_name}", :green
          release.update_attributes(
            release_nzb_downloaded: true,
            release_nzb_download_method: "inner-sanctum #{hash_regex.match(link.href)}",
            release_nzb_location: nzb
          )
          end
        end

        if !release.release_nzb_downloaded && results_page.links_with(:text => 'Download', :href => town_regex).any? do |link|
          agent.get(link.click).save! nzb
          say "Town = #{release.release_name}", :green
          release.update_attributes(
            release_nzb_release_nzb_downloaded: true,
            release_nzb_download_method: "Town",
            release_nzb_location: nzb
          )
          end
        end

        if !release.release_nzb_downloaded && results_page.links_with(:text => 'Download', :href => hash_regex).any? do |link|
          search_form.q = hash_regex.match(link.href)
          results_page  = agent.submit(search_form, search_form.buttons.first)
          form          = results_page.form_with(:action => 'http://nzbindex.nl/download/')
          form.checkboxes.each { |checkbox| checkbox.check if hash_regex.match(checkbox.text) }
          form.submit.save! nzb
          say "#{hash_regex.match(link.href)} = #{release.release_name}", :green
          release.update_attributes(
            release_nzb_downloaded: true,
            release_nzb_download_method: "hash_regex #{hash_regex.match(link.href)}",
            release_nzb_location: nzb
          )
          end
        end

        if !release.release_nzb_downloaded && results_page.links_with(:text => 'Download', :href => psydealer_regex).any? do |link|
          agent.get(link.click).save! nzb
          say "Psydealer = #{release.release_name}", :green
          release.update_attributes(
            release_nzb_downloaded: true,
            release_nzb_download_method: "Psydealer",
            release_nzb_location: nzb
          )
          end
        end

        if !release.release_nzb_downloaded && results_page.links_with(:text => 'Download', :href => flac_regex).each do |link|
          form = results_page.form_with(:action => 'http://nzbindex.nl/download/')
          form.checkboxes.each { |checkbox| checkbox.check if flac_regex.match(link.href) }
          form.submit.save! nzb
          say "FLAC = #{release.release_name}", :green
          release.update_attributes(
            release_nzb_downloaded: true,
            release_nzb_download_method: "FLAC",
            release_nzb_location: nzb
          )
          end
        end

        if !release.release_nzb_downloaded
          release.update_attribute(:release_nzb_download_attempts, (release.release_nzb_download_attempts + 1))
          say "NOPE = #{release.release_name}", :red
        end

        # ToDo: from here down, rewrite.

        if release.release_nzb_downloaded
          nfo_agent = Mechanize.new { |agent| agent.user_agent_alias = 'Mac Safari' }
          nfo_page  = nfo_agent.get("http://nfodb.net.ru/get_nfo-#{release.release_number}.php")
          nfo_page.save! nfo
          say ".nfo created - #{release.release_name}", :green
          release.update_attributes(
            release_nfo_downloaded: true,
            release_nfo_location: nfo
          )
        end

        if release.release_nzb_downloaded && release.release_nfo_downloaded
          system("zip -vjr #{zip} #{download_location}/* -x \"*.DS_Store\"" )
          release.update_attributes(
            release_zip_location: zip,
            release_zip_created: true
          )
          say ".zip created - #{release.release_name}", :green
        end

        if release.release_zip_created
          reply = './etc/reply.txt'
          system("curl -kF \"file=@#{zip}\" https://anonfiles.com/api/v1/upload -o \"#{reply}\"")
          zip_location = IO.readlines(reply)[0]
          anon_hash    = /\b[A-Za-z0-9]{32}\b/.match(zip_location)
          release.update_attribute(:release_zip_uploaded_url, "https://anonfiles.com/file/#{anon_hash}")
          say "#{release.release_name} = https://anonfiles.com/file/#{anon_hash}", :green
          File.open('./etc/gist.txt', 'a') { |f| f.write "* #{release.release_name} = https://anonfiles.com/file/#{anon_hash}\n" }
          File.truncate(reply, 0)
        end
      end

      def gist_tweet_releases
        unless File.zero?('./etc/gist.txt')
          system("gist -c -p -d \"#{DateTime.now}\" -f \"#{DateTime.now}\" './etc/gist.txt' -f download.thor './etc/download.thor'")
          system("t set active \"#{Settings.twitter_username}\"")
          system("t update `pbpaste`")
          File.truncate('./etc/gist.txt', 0)
        end
      end
    end

    desc 'open_nzbs GENRE AUDIO_FORMAT', 'Download a specific genre\'s .nzb\'s. Defaults to FLAC and MP3'
    def open_nzbs(release_genre_formatted, release_audio_format="MP3|FLAC")
      Release.where(
        release_audio_format: %r(#{release_audio_format}),
        release_genre_formatted: release_genre_formatted,
        release_nzb_opened: false
      ).each do |release|
        system("open #{release.release_nzb_location}")
        release.update_attribute(:release_nzb_opened, true)
        say "#{release.release_name}", :cyan
      end
    end

  end
end


