module Autoaudio
  class Make < Thor
  include Autoaudio::Utils

    desc 'releases', 'make all releases'
    method_option :initial, :type => :boolean, :aliases => "-i"
    def releases
      if options.initial?
        if yes?("This could take quite a while. Are you sure? [yN]")
          say "Making Genres...", :white
          say ""
          say ""
          genres
          say "Making Release Dates...", :white
          say ""
          say ""
          release_dates
          say "Making Releases...", :white
          say ""
          say ""
          ReleaseDate.each { |release_date| parse_single_page(release_date.release_date_url) }
        end
      end
      say "Parsing Latest Feed...", :white
      say ""
      say ""
      parse_feed
      if options.initial?
        say "Phew..."
      end
    end

    desc 'genres', 'makes all genres'
    def genres
      doc = Nokogiri::HTML(open('http://nfodb.net.ru/index.php?genreid=0'))
      doc.xpath('//*[@id="content"]/div[1]/form/select').children.map do |child|
        unless Genre.where(genre_name: child.text).exists?
          genre = Genre.create(
            :genre_name           => child.text,
            :genre_number         => child['value']
          )
          say "#{genre}", :green
        end
      end
    end

    desc 'release_dates', 'makes all release dates'
    def release_dates
      doc = Nokogiri::HTML(open("http://nfodb.net.ru/"))
      doc.xpath('//*[@id="content"]/form[2]/select').children.map do |child|
        unless ReleaseDate.where(release_date_string: child.text).exists?
          release_date_string = child.text
          r_date              = Date.parse(child.text.gsub(/\./, '-'))
          max_r_date          = Date.today - Settings.maximum_retention
          if r_date > max_r_date
            release_date = ReleaseDate.create(
              :release_date_string => release_date_string,
              :release_date        => r_date
            )
            say "#{release_date}", :green
          end
        end
      end
    end

    no_commands do
      def parse_single_page(url="http://nfodb.net.ru/")
        doc = Nokogiri::HTML(open(url))
        release_row = doc.css("html > body > div:nth-of-type(4) > div > table > tr").map do |r|
          unless r.css('td:nth-of-type(2) > a').empty?
            unless Release.where(release_name: r.css('td:nth-of-type(2) > a').text).exists?
              release = Release.create(
                :release_name              => r.css('td:nth-of-type(2) > a').text,
                :release_date              => Date.parse(r.css('td:nth-of-type(1) > b').text.gsub(/\./, '-')),
                :release_genre             => r.css('td:nth-of-type(3) > a').text,
                :release_number            => r.css('td:nth-of-type(5) > a').map { |a| a['href'].match(/get_nfo-(.*?)\.php/)[1] }.join
              )
            say "#{release}", :green
            end
          end
        end
      end

      def parse_feed
        doc = Nokogiri::XML(open("http://nfodb.net.ru/rss.php"))
        doc.search('item').map do |doc_item|
          unless Release.where(release_name: doc_item.at('title').text.gsub(/\[(.*?)\]./, '')).exists?
            release = Release.create(
              :release_name    => doc_item.at('title').text.gsub(/\[(.*?)\]./, ''),
              :release_date    => Date.parse(doc_item.at('description').text.gsub(/Date:./,'').gsub(/\./, '-')),
              :release_genre   => doc_item.at('title').text[/[^\[]+[^\\]\]/].gsub(/[\[(.*?)\]]/,''),
              :release_number  => doc_item.at('link').text.match(/(http:\/\/nfodb.net.ru\/nfo-)(.*?)(.html)/)[2]
            )
            say "#{release}", :green
          end
        end
      end
    end


  end
end
